package org.eclipse.palamedes.kif.core.ast;


public class KIFConjunction extends KIFLogSent {

    private final KIFSeq<KIFSent> operands = new KIFSeq<KIFSent>();

    KIFConjunction(AST ast) {
        super(ast);
    }

    @Override
    public String getName() {
        return "and";
    }

    @Override
    public KIFSeq<KIFSent> getOperands() {
        return operands;
    }

    @Override
    ASTNode clone0(AST ast) {
        KIFConjunction newNode = ast.newConjunction(operands.clone0(ast));
        newNode.copyMembers(this);
        return newNode;
    }

    @Override
    final int getNodeType0() {
        return ASTNode.CONJUNCTION;
    }

    @Override
    public void traverse(ASTVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    boolean subtreeMatch0(ASTMatcher matcher, Object other) {
        return matcher.match(this, other);
    }

    @Override
    public int getOperandCount() {
        return operands.size();
    }

    protected final void copyMembers(KIFConjunction node) {
        super.copyMembers(node);
        operands.addAll(node.operands);
    }
}
