package org.eclipse.palamedes.kif.core.ast;

public class KIFDisjunction extends KIFLogSent {

    private final KIFSeq<KIFSent> operands = new KIFSeq<KIFSent>();

    KIFDisjunction(AST ast) {
        super(ast);
    }

    @Override
    public String getName() {
        return "or";
    }

    @Override
    public KIFSeq<KIFSent> getOperands() {
        return operands;
    }

    @Override
    ASTNode clone0(AST ast) {
        KIFDisjunction newNode = ast.newDisjunction(operands.clone0(ast));
        newNode.copyMembers(this);
        return newNode;
    }

    @Override
    final int getNodeType0() {
        return ASTNode.DISJUNCTION;
    }

    @Override
    public void traverse(ASTVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    boolean subtreeMatch0(ASTMatcher matcher, Object other) {
        return matcher.match(this, other);
    }

    @Override
    public int getOperandCount() {
        return operands.size();
    }

    protected final void copyMembers(KIFDisjunction node) {
        super.copyMembers(node);
        operands.addAll(node.operands);
        operands.trimToSize();
    }

}
