package org.eclipse.palamedes.kif.core.ast;

public class KIFNegation extends KIFLogSent {

    private final KIFSeq<KIFSent> operands = new KIFSeq<KIFSent>();

    KIFNegation(AST ast) {
        super(ast);
    }

    @Override
    public final KIFSeq<KIFSent> getOperands() {
        return operands;
    }

    @Override
    public String getName() {
        return "not";
    }

    @Override
    ASTNode clone0(AST ast) {
        KIFNegation newNode = ast.newNegation(
                                        (KIFSent) operands.get(0).clone(ast) );
        newNode.setSourceStart ( getSourceStart()  );
        newNode.setSourceLength( getSourceLength() );
        return newNode;
    }

    @Override
    final int getNodeType0() {
        return ASTNode.NEGATION;
    }

    @Override
    public void traverse(ASTVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    boolean subtreeMatch0(ASTMatcher matcher, Object other) {
        return matcher.match(this, other);
    }

    @Override
    public String toString() {
        return "(not " + operands + ")";
    }

    public KIFSent getOperand() {
        return operands.get(0);
    }

    @Override
    public int getOperandCount() {
        return 1;
    }

    protected final void copyMembers(KIFNegation node) {
        super.copyMembers(node);
        operands.addAll(node.operands);
    }

}
