package org.eclipse.palamedes.kif.core.ast;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.palamedes.kif.core.ast.visitor.NotationWriter;

public class KIFSeq<T extends ASTNode> extends ArrayList<T> {

    private static final long serialVersionUID = 2314147446255930877L;

    @Override
    public String toString() {
    	return getInfixNotation();
    }

    @SuppressWarnings("unchecked")
    public KIFSeq<T> clone0(AST ast) {
        KIFSeq<T> seq = new KIFSeq<T>();
        for (ASTNode node : this)
            seq.add( (T) node.clone(ast) );
        return seq;
    }


    public void replace(T oldChild, T newChild) {

        // find index
        int index = indexOf(oldChild);

        // do nothing if the original sentence is not in the form
        if (index == -1)
            return;

        // remove original and add the replacement on the same index;
        remove(index);
        add(index, newChild);

    }

    public void replace(T oldChild, List<T> newChilds) {

        // find index
        int index = indexOf(oldChild);

        // do nothing if the original sentence is not in the form
        if (index == -1)
            return;

        // remove original and add the replacement on the same index;
        remove(index);
        for (int i = 0; i < newChilds.size(); i++)
            add(index + i, newChilds.get(i));
    }
    
	/**
	 * @return the prefix form of this term
	 * e.g., "f(a,g(1,2))" 
	 */
	public String getPrefixNotation(){
		return new NotationWriter(NotationWriter.PREFIX).toString(this, ",");
	}

	/**
	 * @return the infix KIF form of this term
	 * e.g., "(f a (g 1 2))" 
	 */
	public String getInfixNotation(){
		return new NotationWriter(NotationWriter.INFIX).toString(this);
	}
}
