package org.eclipse.palamedes.kif.core.knowledgebase;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import org.eclipse.palamedes.kif.core.ast.KIFForm;
import org.eclipse.palamedes.kif.core.knowledgebase.filter.FunctionFilter;
import org.eclipse.palamedes.kif.core.knowledgebase.filter.RelationFilter;

public class KnowledgeBase {

    protected KIFForm                  description;
    protected TreeMap<String, KBEntry> entries;

    /**
     * The setDescription method resets the knowledge base and sets the new
     * GDL description.
     * @param description
     */
    public void setDescription(KIFForm gdl) {
        if (gdl == null || gdl.isEmpty())
            return;

        description = gdl;
        entries     = new KnowledgeBaseVisitor(description).getEntries();
    }

    /**
     * Returns the KBEntry object given by name.
     */
    public KBEntry get(String name) {
        return entries.get(name);
    }

    /**
     * Returns all KBEntry objects in a list.
     */
    public List<KBEntry> getEntries() {
        return new ArrayList<KBEntry>( entries.values() );
    }

    /**
     * Returns a list of KBEntry objects that are satisfying the constrains
     * given by filter.
     */
    public List<KBEntry> filter(IKBFilter filter) {
    	ArrayList<KBEntry> filtered = new ArrayList<KBEntry>();
        for ( KBEntry entry : entries.values() )
            if ( filter.isMatching(entry) )
                filtered.add(entry);
        return filtered;
    }

    /*
     * (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("Knowledge Base");

        sb.append("\n\nFunctions:");
        for (KBEntry info : filter(new FunctionFilter()))
            sb.append("\n").append(info.toString().replace("\n", "\n\t"));

        sb.append("\n\nRelations:");
        for (KBEntry info : filter(new RelationFilter()))
            sb.append("\n").append(info.toString().replace("\n", "\n\t"));

        return sb.toString();
    }

}
