package org.eclipse.palamedes.kif.core.parser;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import java_cup.runtime.lr_parser;

import org.eclipse.palamedes.kif.core.ast.AST;

/**
 * This is just a convenience class for all parsers generated by cups.
 * @author Ingo Keller
 *
 */
public class Parser {

    /* Parser Support */
    final lr_parser parser;

    /**
     * Default constructor
     */
    public Parser() {
        parser = new KIFParser();
    }

    /**
     * Constructor that uses a special AST.
     */
    public Parser(AST ast) {
        parser = new KIFParser(ast);
    }

    /**
     * Parses a file containing a GDL description into an expression list.
     * @param gdl
     * @return
     */
    public Object parseFile( String filename ) {
        try {
            return parse( new FileInputStream( filename ) );
        } catch ( FileNotFoundException exception ) {
            exception.printStackTrace();
            return null;
        }
    }

    /**
     * Parses a string containing a GDL description into an expression list.
     * @param gdl
     * @return
     */
    public Object parseGDL( String gdl ) {
        return parse( new ByteArrayInputStream( gdl.getBytes() ) );
    }

    /**
     * Parses a input stream into a expression list object.
     * Be aware: This is the real core parsing method.
     * @param stream
     * @return
     */
    private Object parse( InputStream stream ) {
        try {
            parser.setScanner( new KIFScanner( stream ) );
            return parser.parse().value;
        } catch ( Exception exception ) {

            // TODO: Better error handling
            // exception.printStackTrace();
            return null;
        }
    }

}
